/***********************************************************
 * $Id: fancyarray.h 1808 2016-12-31 22:56:01Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * An allocated array.
 *
 **************************************************************/

#ifndef FANCYBASE_ARRAY_H
#define FANCYBASE_ARRAY_H

#include <fancyobject.h>

/**
 * Pass a Fancy::ArrayObject to classical C-style function parameters
 * void *, size_t e.g. to crypto API or I/O functions.
 */
#define FANCY_ARRAY_OBJ_C_ARGS(ao) (ao).isNull()?NULL:(ao)->elem_p(0),(ao).isNull()?(size_t)0:(ao)->getSize()

namespace Fancy {

  /**
   * If there is no need for a full blown ArrayObject<uint8_t>::ptr_t,
   * Array<uint8_t> can be used as leak- and exception-safe byte[] replacement
   */
  template<class T> class Array
  {
    T *t;

  public:

    Array(size_t sz) : t(new T[sz]) {}

    ~Array() { delete[] t; }

    const T &operator[] (size_t i) const { return t[i]; }
    T &operator[] (size_t i) { return t[i]; }

    const T &elem(size_t i) const { return t[i]; }
    T &elem (size_t i) { return t[i]; }

    const T * elem_p(size_t i) const { return t+i; }
    T * elem_p(size_t i) { return t+i; }
  };

  template<class T> class ArrayObject : public Array<T>, virtual public Object
  {
      size_t sz;

  public:
      typedef T *iterator;
      typedef const T *const_iterator;
      typedef Fancy::ObjPtr<ArrayObject<T> > ptr_t;

      static ptr_t newInstance(size_t _sz) { return new ArrayObject(_sz); }

      size_t getSize() const { return sz; }

      iterator begin() { return this->elem_p(0); }
      iterator end() { return this->elem_p(sz); }

      /**
       * For primitive types like uint8_t this can be used to get a const char* to the begin of the backing memory block
       */
      const_iterator begin() const { return this->elem_p(0); }

      /**
       * For primitive types like uint8_t this can be used to get a const char* to the last element in the backing memory block
       */
      const_iterator end() const { return this->elem_p(sz); }

      bool lessThan(const Object &o) const {

          const ArrayObject *ao = dynamic_cast<const ArrayObject *>(&o);
          if (!ao) return this->uncomparableLessThan(o);

          if (this->getSize() < ao->getSize()) return true;
          if (this->getSize() > ao->getSize()) return false;

          for (size_t i=0; i< this->getSize();++i) {
              if (this->elem(i) < ao->elem(i)) return true;
              if (ao->elem(i) < this->elem(i)) return false;
          }
          return false;
      }

      bool equals(const Object &o) const {

          const ArrayObject *ao = dynamic_cast<const ArrayObject *>(&o);
          if (!ao) return false;

          if (this->getSize() != ao->getSize()) return false;

          for (size_t i=0; i< this->getSize();++i) {
              if (this->elem(i) != ao->elem(i)) return false;
          }
          return true;
      }

  protected:
      ArrayObject(size_t _sz) : Array<T>(_sz),sz(_sz)
      {}

      ~ArrayObject() {}
  };

  inline ArrayObject<int8_t>::ptr_t to_signed(const ArrayObject<uint8_t>::ptr_t &o) {
      return ArrayObject<int8_t>::ptr_t(reinterpret_cast<ArrayObject<int8_t>*>(o.get()));
  }

  inline ArrayObject<uint8_t>::ptr_t to_unsigned(const ArrayObject<int8_t>::ptr_t &o) {
      return ArrayObject<uint8_t>::ptr_t(reinterpret_cast<ArrayObject<uint8_t>*>(o.get()));
  }

  inline ArrayObject<int16_t>::ptr_t to_signed(const ArrayObject<uint16_t>::ptr_t &o) {
      return ArrayObject<int16_t>::ptr_t(reinterpret_cast<ArrayObject<int16_t>*>(o.get()));
  }

  inline ArrayObject<uint16_t>::ptr_t to_unsigned(const ArrayObject<int16_t>::ptr_t &o) {
      return ArrayObject<uint16_t>::ptr_t(reinterpret_cast<ArrayObject<uint16_t>*>(o.get()));
  }

  inline ArrayObject<int32_t>::ptr_t to_signed(const ArrayObject<uint32_t>::ptr_t &o) {
      return ArrayObject<int32_t>::ptr_t(reinterpret_cast<ArrayObject<int32_t>*>(o.get()));
  }

  inline ArrayObject<uint32_t>::ptr_t to_unsigned(const ArrayObject<int32_t>::ptr_t &o) {
      return ArrayObject<uint32_t>::ptr_t(reinterpret_cast<ArrayObject<uint32_t>*>(o.get()));
  }

  inline ArrayObject<int64_t>::ptr_t to_signed(const ArrayObject<uint64_t>::ptr_t &o) {
      return ArrayObject<int64_t>::ptr_t(reinterpret_cast<ArrayObject<int64_t>*>(o.get()));
  }

  inline ArrayObject<uint64_t>::ptr_t to_unsigned(const ArrayObject<int64_t>::ptr_t &o) {
      return ArrayObject<uint64_t>::ptr_t(reinterpret_cast<ArrayObject<uint64_t>*>(o.get()));
  }

}

#endif
