/***********************************************************
 * $Id: fancybase64bufferchannel.h 1892 2017-10-25 09:51:32Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Author: Christoph Lechleitner/IBCL <lech@ibcl.at>
 *
 * Header of Base64BufferChannel
 * A channel reading/writing from/to in-RAM buffers
 *
 ************************************************************/

#ifndef FANCY_BASE64BUFFERCHANNEL_H_
#define FANCY_BASE64BUFFERCHANNEL_H_

#include <fancychannel.h>
#include <fancyobject.h>

namespace Fancy {

template<class T> class ArrayObject;

/*!
 * Base class for Channels that operate on an in-memory buffer and decodes or encodes base64 when writing on it.
 * Also allows direct read acces to the decoded/encoded for easier work on short data blocks like for HTTP Basic Authentication.
 *
 * Base64: https://tools.ietf.org/html/rfc4648
 * URL safe variant: https://tools.ietf.org/html/rfc4648#section-5
 */
class FANCYBASE_API Base64BufferChannel : public virtual Channel {

public:
    typedef ObjPtr<Base64BufferChannel> ptr_t;

    /** returns a Base64BufferChannel instance decoding data on write().
     */
    static ptr_t newDecodingInstance(int bufsize);

    /** returns a Base64BufferChannel instance encoding data on write().
     */
    static ptr_t newEncodingInstance(int bufsize);

    /**
     * returns a Base64BufferChannel instance encoding data on write().
     *
     * urlSafe: wether to write URL safe variant ac. https://tools.ietf.org/html/rfc4648#section-5
     * URL safety recomends to suppress padding.
     *
     * withPadding: wether to write the padding appendix
     * Suppressing padding requires other clear end-of-data markers like " or URL query structure.
     *
     * rowWidth: maximum width of text rows. 0 or -1 suppress line breaks.
     */
    static ptr_t newEncodingInstance(int bufsize, bool urlSafe, bool withPadding, int rowWidth=64);

    /** copies count bytes of the buffer to data which must have count bytes reserved */
    virtual void getData(void *data, int count) const = 0;

    /** returns a pointer to the data buffer. very dangerous, but useful for i.e. cheap logging. */
    virtual uint8_t* getDataPtr() const = 0;

    /** returns the current read position */
    virtual int getReadPos() const = 0;

    /** returns the current write position */
    virtual int getWritePos() const = 0;

};


}

#endif /* FANCY_BASE64BUFFERCHANNEL_H_ */
