/***********************************************************
 * $Id: fancybufferchannel.h 1828 2017-04-12 13:23:24Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Author: Christoph Lechleitner/IBCL <lech@ibcl.at>
 *
 * Header of BufferChannel
 * Channels reading/writing from/to in-RAM buffers with base64 encoding/decoding on write operations
 *
 ************************************************************/

#ifndef FANCY_BUFFERCHANNEL_H_
#define FANCY_BUFFERCHANNEL_H_

#include <fancychannel.h>
#include <fancyobject.h>

namespace Fancy {

template<class T> class ArrayObject;

/*!
 * A Channel that operates on an in-memory buffer, i.e. a uint8_t*,
 * and allows direct access to this buffer.
 */
class FANCYBASE_API BufferChannel : public virtual Channel {

public:
    typedef ObjPtr<BufferChannel> ptr_t;

    /**
     * Returns a BufferChannel instance operating on a buffer with a fixed size of
     * bufsize bytes.
     *
     * If @c bufsize is negative, the buffer can grow beyond the magnitude of the
     * given parameter, just like a std::ostringstream.
     *
     * Initially data can only be written (via Channel methods), unless set with setData()/appendData().
     */
    static ptr_t newInstance(int bufsize);

    /**
     * Copies up to count bytes from data into the buffer and sets writePos to the end of the written data, sets readPos to 0.
     *
     * Checks bufsize quietly, i.e. does not crash nor throw an exception, just swallows excessive data.
     *
     * setData(0,0) safely resets the buffer.
     */
    virtual void setData(const void *data, int count) = 0;

    /**
     * Copies up to count bytes from data into the buffer and sets writePos to the end of the written data, sets readPos to 0.
     *
     * Checks bufsize quietly, i.e. does not crash nor throw an exception, just swallows excessive data.
     *
     * SetData(0,0) safely resets the buffer.
     */
    virtual void setData(const ObjPtr<ArrayObject<int8_t> > &data, int count = -1) = 0;

    /**
     * Sets buffer[0-count] to fillByte, sets writePos to count, readPos to 0.
     *
     * Checks bufsize quietly, i.e. does not crash nor throw an exception, just ignores excessive fill wishes.
     */
    virtual void fillBuffer(int count, uint8_t fillByte=0) = 0;

    /**
     * Copies up to count bytes from data to the buffer[writePos] and sets writePos to the end of the written data, does not touch readPos.
     *
     * Checks bufsize quietly, i.e. does not crash nor throw an exception, just swallows excessive data.
     * Use getFreeSpace() and realignDroppingReadData() to avoid data swallowing.
     */
    virtual int write (const void *buf, int count) = 0;

    /**
     * Copies up to count bytes from data to the buffer[writePos] and sets writePos to the end of the written data, does not touch readPos.
     *
     * Checks bufsize quietly, i.e. does not crash nor throw an exception, just swallows excessive data.
     * Use getFreeSpace() and realignDroppingReadData() to avoid data swallowing.
     */
    virtual int writeFromArray(const ObjPtr<ArrayObject<int8_t> > &data, int count = -1) = 0;

    /**
     * Reads up to count bytes from fromChannel and write them to buffer[writePos..],
     * and sets writePos to the end of the written data, does not touch readPos.
     *
     * Checks bufsize quietly, i.e. does not crash nor throw an exception, just swallows excessive data.
     * Use getFreeSpace() and realignDroppingReadData() to avoid data swallowing.
     *
     * If maxCount is specified negative, getFreeSpace() is used.
     *
     * Returns the number of bytes actually read, i.e. forwards the answer from fromChannel->read()
     */
    virtual int writeFromChannel(const Fancy::ObjPtr<Channel> & fromChannel, int maxCount=-1) = 0;

    /**
     * "Realigns" the buffer by "moving" the "real data" (from readPos to writePos) to the left.
     * Data already read, i.e. from 0 to readPos, are thrown away.
     *
     * readPos and writePos are updated accordingly.
     *
     * Returns the number of read bytes discarded, i.e. the length of the realignment.
     */
    virtual int realignDroppingReadData() = 0;

    /**
     * Copies up to count bytes from the buffer to *buf, and moves the readPos accordingly.
     */
    virtual int read(void *buf, int count) = 0;

    /**
     * Copies up to maxCount bytes from the buffer to the specified channel, and moves the readPos accordingly.
     *
     * If count is specified negative, all written but unread data are forwarded.
     *
     * If count is higher that writePos-readPos, only writePos-readPos bytes are forwared.
     *
     * In either case only ONE call toChannel->write(void*, maxCount) is made.
     *
     * Returns the number of bytes actually forwarded.
     */
    virtual int readToChannel(const Fancy::ObjPtr<Channel> & toChannel, int maxCount = -1) = 0;

    /**
     * Returns a pointer to the start of the data buffer (resp. the offset'th byte).
     * Very dangerous, but useful for i.e. cheap logging.
     */
    virtual uint8_t* getDataPtr(int offset = 0) const = 0;

    /**
     * Copies count bytes from the start of the buffer to *data which must have count bytes reserved.
     */
    virtual int getData(void *data, int count) const = 0;

    /**
     * Returns the current read position
     */
    virtual int getReadPos() const = 0;

    /**
     * Returns the current write position
     */
    virtual int getWritePos() const = 0;

    /**
     * Returns size-writePos, i.e. tells how many data may be appended
     */
    virtual int getFreeSpace() const = 0;

    /**
     * Searches the buffer[readPos..writePos] for the value c and.
     * Returns the position of the first occurrence, or -1 if it has not been found
     */
    //virtual int search(uint8_t c) const = 0;

    // remaining methods inherited from Channel
    virtual void flush() = 0;
    virtual void close() = 0;
    virtual bool isOpen() const = 0;
    virtual void shutdown(ShutdownOption o) = 0;

    // FIXME fancylibs4: make growOrLimit (or a new grow-only method public:
    /*
     * Grow the buffer or reduce @c n to the number of
     * remaining buffer size by concerning @c pos as the
     * insertion point for the new data.
     *
     * If pos < 0, getWritePos() is used
     */
    //int growOrLimit(int pos, int n);


};


}

#endif /* FANCY_BUFFERCHANNEL_H_ */
