/***********************************************************
 * $Id: fancyconfig.h 1572 2015-03-08 20:10:55Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** Class for parsing command line arguments and config files.
**
*****************************************************************************/

#ifndef FANCYBASE_CONFIG_H
#define FANCYBASE_CONFIG_H

#include <fancyobject.h>

namespace Fancy {

class Map;

  class FANCYBASE_API ConfigOptions: virtual public Object
  {
  public:
      typedef ObjPtr<ConfigOptions> ptr_t;

      static ptr_t newInstance(const std::string &ctxt);

      /**
       * Loads configuration options from a text file with name=value lines.
       *
       * If I/O errors occur or if more than syntaxErrorsAllowed lines are malformed, a Fancy::Exception is thrown.
       *
       * Empty lines are allowed.
       * White space is stripped basically, but is respected within names or values.
       * Lines with # or ; as first non-white character are considered comments.
       */
      virtual void loadFile(const std::string &fn, int syntaxErrorsAllowed = 0) = 0;

      virtual void parseCmdLine(int &argc, char **argv) = 0;

      virtual void setValue (const std::string &nm, const std::string &v) = 0;
      virtual void setBool  (const std::string &nm, bool v) = 0;
      virtual void setInt   (const std::string &nm, int v) = 0;
      virtual void setUInt  (const std::string &nm, unsigned v) = 0;
      virtual void setDouble(const std::string &nm, double v) = 0;
      virtual void setInt64  (const std::string &nm, int64_t v) = 0;
      virtual void setUInt64 (const std::string &nm, uint64_t v) = 0;

      virtual std::string getValue (const std::string &nm, const std::string &def) const = 0;
      virtual bool        getBool  (const std::string &nm, bool def) const = 0;
      virtual int         getInt   (const std::string &nm, int def) const = 0;
      virtual unsigned    getUInt  (const std::string &nm, unsigned def) const = 0;
      virtual double      getDouble(const std::string &nm, double def) const = 0;
      virtual int64_t     getInt64  (const std::string &nm, int64_t def) const = 0;
      virtual uint64_t    getUInt64 (const std::string &nm, uint64_t def) const = 0;

      virtual std::string getValue (const std::string &nm) const = 0;
      virtual bool        getBool  (const std::string &nm) const = 0;
      virtual int         getInt   (const std::string &nm) const = 0;
      virtual unsigned    getUInt  (const std::string &nm) const = 0;
      virtual double      getDouble(const std::string &nm) const = 0;
      virtual int64_t     getInt64  (const std::string &nm) const = 0;
      virtual uint64_t    getUInt64 (const std::string &nm) const = 0;

      static std::string getHostName();

      /**
       * Loads all matching files in the specified directory.
       * If filenamePattern is empty, all files are read. Otherwise Fancy::Directory::findEntries(filenamePattern) is used.
       * Files starting with '.' are ignored. This should cover temporary files from editors.
       * Subdirectories are ignored.
       */
      virtual void loadDirectory(
              const std::string &dirName,
              const std::string &filenamePattern,
              int errorsAllowed = 0) = 0;

      /**
       * Returns the full path of the binary (the .exe file)
       */
      static std::string getBinaryPath();

      /**
       * Get the native error message from a native error code.
       *
       * Under UNIX, this return std::string(strerror(errorCode))
       *
       * Under Win32, return the result of FormatMessageW()for the given Win32 error code
       * as an UTF-8 encoded string.
       *
       * For HRESULT result codes, you might do
       *
       * if (HRESULT_FACILITY(hresult) == FACILITY_WIN32)) {
       *    errMsg = Fancy::CException::getWin32ErrorMessage(HRESULT_CODE(hresult));
       * }
       */
      static std::string getNativeErrorMessage(int errorCode);

  protected:
    ConfigOptions();
    virtual ~ConfigOptions();
  };


  class FANCYBASE_API INIParser: virtual public Object
  {
  public:
      typedef ObjPtr<INIParser> ptr_t;

      static ptr_t newInstance(const std::string &ctxt);

      /**
       * Loads multiple ConfigOptions objects from a text file with [section] lines and name=value lines.
       *
       * For each [section] a ConfigOptions object is created, using this objects' context + '.' + section as context.
       *
       * See ConfigOptions.loadFile for syntax rules.
       *
       * Returns a map using section names (Fancy::String::ptr_t) as keys, ac. configOptions as values
       */
      virtual Fancy::ObjPtr<Fancy::Map> parseINIFile(const std::string &fn, int syntaxErrorsAllowed = 0) = 0;

  protected:
      INIParser();
      virtual ~INIParser();
  };


}

#endif // ! FANCYBASE_CONFIG_H
