/***********************************************************
 * $Id: fancydeflatechannel.h 1810 2017-01-13 14:49:58Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Author: Christoph Lechleitner/IBCL <lech@ibcl.at>
 *
 * Header of BufferChannel
 * Channels reading/writing from/to in-RAM buffers with base64 encoding/decoding on write operations
 *
 ************************************************************/

#ifndef FANCY_DEFLATECHANNEL_H_
#define FANCY_DEFLATECHANNEL_H_

#include <fancychannel.h>

namespace Fancy {

/**
 * A Channel that uses zlib to apply deflate/inflate compression/decompression
 * to streams.
 *
 * This implementation is thread-safe in the sense, that separate threads may
 * call read() and write() and yet-another thread my call close() or shutdown().
 * Concurrent read() or write() operations from two threads are not supported
 * and will most likely lead to unpredictable results.
 *
 * Please note, that you have to call close() or flush() to really finish the
 * generated output stream.
 *
 * After calling flush(), a new deflated stream may be started by writing more
 * data to this channel.
 */
class FANCYBASE_API DeflateChannel : public virtual Channel {

public:
    typedef ObjPtr<DeflateChannel> ptr_t;

    /**
     * Returns a BufferChannel instance operating on a buffer with bufsize bytes
     * and the given default compression.
     *
     * The default buffer size and default compression should be suitable
     * for most purposes.
     *
     * If defaultCompression is set to a negative number in the range -1 to -9,
     * A gzip-compatible stream according to RFC1952 is written instead of a
     * deflate-stream according to RFC1951.
     *
     * Upon reading, you also need to pass a negative defaultCompression in order to
     * decode an RFC1952 gzip stream instead of the default RFC1951 deflate-stream
     * format.
     *
     * HTTP response bodies are RFC1951-encoded, if the @c Content-Encoding header
     * is set to @c deflate. If the @c Content-Encoding header is set to
     * @c gzip, the response body is RFC1952-encoded.
     */
    static ptr_t newInstance(const Fancy::ObjPtr<Fancy::Channel> &channel, int defaultCompression=9, int bufsize=16384);

    //TODO
    static ptr_t newRawInstance(const Fancy::ObjPtr<Fancy::Channel> &channel, int defaultCompression=9, int bufsize=16384);

    /**
     * Returns the total amount of bytes written so far, after compression
     *
     * Call this method before calling flash, because flush resets the value to 0
     */
    virtual uint64_t getWrittenCompressedSize() = 0;

    /**
     * Returns the total amount of bytes written so far, before compression
     *
     * Call this method before calling flash, because flush resets the value to 0
     *
     */
    virtual uint64_t getWrittenUncompressedSize() = 0;

    /**
     * Returns the crc32 checksum of all bytes written so far
     *
     * Call this method before calling flash, because flush resets the value to the "magic number"
     */
    virtual uint32_t getWrittenCrc32() = 0;

    /**
     * Returns the total amount of bytes read so far, after compression
     *
     * Call this method before calling flash, because flush resets the value to 0
     */
    virtual uint64_t getReadCompressedSize() = 0;

    /**
     * Returns the total amount of bytes read so far, before compression
     *
     * Call this method before calling flash, because flush resets the value to 0
     *
     */
    virtual uint64_t getReadUncompressedSize() = 0;

    /**
     * Returns the crc32 checksum of all bytes read so far
     *
     * Call this method before calling flash, because flush resets the value to the "magic number"
     */
    virtual uint32_t getReadCrc32() = 0;

protected:
    DeflateChannel();
    ~DeflateChannel();
};

}

#endif /* FANCY_DEFLATECHANNEL_H_ */
