/***********************************************************
 * $Id: fancyisocountry.h 1227 2014-04-23 11:55:12Z fancylibs $
 *
 * FancyLibs ISO Country stuff
 *
 * Copyright (C) 2014 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCY_ISOCOUNTRY_H_
#define FANCY_ISOCOUNTRY_H_

#include <fancyobject.h>


namespace Fancy {


/**
 * A ISO country object holds the usualy ISO-3166-1 attributes for a single country.
 */
class FANCYBASE_API ISOCountry : public virtual Fancy::Object {
protected:
    virtual ~ISOCountry();
public:
    typedef ObjPtr<ISOCountry> ptr_t;

    static ISOCountry::ptr_t newInvalidInstance();

    /**
     * Returns Alpha2
     */
    virtual std::string getAlpha2() const = 0;

    /**
     * Returns Alpha3
     */
    virtual std::string getAlpha3() const = 0;

    /**
     * Returns number code
     */
    virtual int getNumber() const = 0;

    /*
     * Returns the name. Usually without candy like 'Republic of ...'
     */
    virtual std::string getName() const = 0;

    /*
     * Returns wether the object looks valid.
     * Mainly used to detect invalid empty objects returned by failed searches.
     */
    virtual bool isValid() const = 0;

};


/**
 * A ISO country map provides means to find ISOCountry instances by any of the usual attributes.
 *
 * It contains a hard coded basic list based on iso_3166.xml from Debian's iso-codes package.
 * Future versions may allow to read text files.
 *
 * Alpha2 and Alpha3 are converted to uppercase when reading files or search for them.
 */
class FANCYBASE_API ISOCountryMap : public virtual Fancy::Object {
protected:
    virtual ~ISOCountryMap();
public:
    typedef ObjPtr<ISOCountryMap> ptr_t;

#ifndef WIN32
    /**
     * Return a new ISOCountryMap instance, based on reading /etc/fancylibs/fancybase3/iso3166-1.txt
     */
    static ptr_t newInstance();
#endif

    /**
     * Return a new ISOCountryMap instance, based on reading filename.
     *
     * The file must contain at least 1 data line and may contain comment lines.
     * Data line contain alpha2, alpha3, numericCode (decimal) and name, separated by tab characters.
     * Comment lines start with #
     * Empty lines are allowed too.
     *
     * FancyLibs packages usually contain an apropriate file as etc/fancylibs/fancybase3/iso3166-1.txt
     */
    static ptr_t newInstance(const std::string & filename);

    /**
     * Loads another data file, overwriting eventual conflicting data already loaded.
     */
    virtual int loadFile(const std::string & filename) = 0;

    /**
     * Clears the country list. For testing and the like.
     */
    virtual void clear() = 0;

    /**
     * Returns the number of countries (actually nr. of alpha2 keys)
     */
    virtual int getNrCountries() const = 0;

    /**
     * Returns the found country, or an invalid empty country.
     */
    virtual ISOCountry::ptr_t getCountryByAlpha2(const std::string & alpha) = 0;

    /**
     * Returns the found country, or an invalid empty country.
     */
    virtual ISOCountry::ptr_t getCountryByAlpha3(const std::string & alpha) = 0;

    /**
     * Returns the found country, or an invalid empty country.
     */
    virtual ISOCountry::ptr_t getCountryByNumber(int nr) = 0;

};


}

#endif /* FANCY_ISOCOUNTRY_H_ */
