/***********************************************************
 * $Id: fancylist.h 1260 2014-06-28 19:27:16Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYLIST_H_
#define FANCYLIST_H_

#include <fancycollection.h>
#include <fancyarray.h>
#include <fancyserializable.h>

namespace Fancy {

class Iterator;

/*!
 * An interface for an untyped list of objects.
 * This is pretty much like \c java.util.List and is used in favor
 * to a template in order to avoid code-bloat and to allow a generic
 * deserializer to be implemented.
 */
class FANCYBASE_API List: public virtual Collection {
public:
    typedef ObjPtr<List> ptr_t;

    virtual size_t getSize() const = 0;

    virtual ObjPtr<Object> get(size_t i) const = 0;
    virtual std::string getString(size_t i) const;

    virtual void set(size_t i, const ObjPtr<Object> &o) = 0;
    virtual void setString(size_t i, const std::string &v);

    virtual ObjPtr<Iterator> getBeginIterator();

    /**
     * Returns an iterator pointing to the element _after_ the last element of the list.
     * Do NOT use this and a for loop to iterate backwards, you'll miss the first element!
     */
    virtual ObjPtr<Iterator> getEndIterator();

    virtual bool isGrowable() const;

    // optional operations.
    virtual void add(const ObjPtr<Object> &o);
    virtual void addString(const std::string &v);

    // removes the object at position i and returns it
    virtual ObjPtr<Object> remove(size_t i);

    // removes the objects from position first to position last
    virtual void remove(size_t first, size_t last);

    virtual std::string removeString(size_t i);
    virtual ObjPtr<Object> pop();
    virtual std::string popString();
    virtual void clear();
    virtual void insert(size_t i, const ObjPtr<Object> &o);
    virtual void insertString(size_t i, const std::string &v);

    bool lessThan(const Object &o) const;
    bool equals(const Object &o) const;

    std::string toString() const;

    static ObjPtr<List> newInstance();
    static ObjPtr<List> newInstance(size_t reserve);
    static ObjPtr<List> newInstance(getSerialID_s_t getSerialID_s, getSerialVersionUID_s_t getSerialVersionUID_s);
    static ObjPtr<List> newInstance(getSerialID_s_t getSerialID_s, getSerialVersionUID_s_t getSerialVersionUID_s, size_t reserve);

    template <class T> static ObjPtr<List> newTypedInstance() {
        return List::newInstance(T::getSerialID_s,T::getSerialVersionUID_s);
    }

    template <class T> static ObjPtr<List> newTypedInstance(size_t reserve) {
        return List::newInstance(T::getSerialID_s,T::getSerialVersionUID_s,reserve);
    }

protected:
    List();
    virtual ~List();
};

typedef ObjPtr<List> (*newArrayObjectListInstance_t)(size_t);

/*!
 * This class adapts the fixed-size ArrayObject template to the untyped List interface,
 * which is mainly needed in order to implement a generic deserializer.
 */
template <class T> class ObjectArray : public ArrayObject<ObjPtr<T> >, public List {

public:
    typedef ObjPtr<ObjectArray<T> > ptr_t;
    typedef ArrayObject<ObjPtr<T> > super_t;

    static ptr_t newInstance(size_t sz)
    { return new ObjectArray(sz); }

    static ObjPtr<List> newListInstance(size_t sz)
    { return new ObjectArray(sz); }

    const char *getComponentSerialID()     const { return T::getSerialID_s(); }
    int64_t getComponentSerialVersionUID() const { return T::getSerialVersionUID_s();}

    size_t getSize() const { return super_t::getSize(); }

    ObjPtr<Object> get(size_t i) const { return ObjPtr<Object>(*super_t::elem_p(i)); }

    void set(size_t i, const ObjPtr<Object> &o) { *super_t::elem_p(i) = o.check_cast<T>(); }

    bool lessThan(const Object &o) const {
        return List::lessThan(o);
    }

    bool equals(const Object &o) const {
        return List::equals(o);
    }

protected:
    ObjectArray(size_t sz) :
        ArrayObject<ObjPtr<T> >(sz) {}
   ~ObjectArray() {}
};

}

#endif /* FANCYLIST_H_ */
