/***********************************************************
 * $Id: fancymap.h 1118 2014-01-15 19:47:08Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYMAP_H_
#define FANCYMAP_H_

#include <fancycollection.h>
#include <fancyserializable.h>

namespace Fancy {

class Iterator;
class List;
class Set;

/*!
 * An interface for an untyped set of objects.
 * This is pretty much like \c java.util.Map and is used in favor
 * to a template in order to avoid code-bloat and to allow a generic
 * deserializer to be implemented.
 */
class FANCYBASE_API Map: public virtual Collection {
public:
    typedef ObjPtr<Map> ptr_t;

    Map() {}

    /**
     * Return the serial ID of the declared key type of this map or
     * @c NULL, if the component type is @c Object.
     */
    virtual const char *getKeySerialID() const = 0;
    /**
     * Return the serialVersionUID of the declared key type of this map or
     * @c (int64_t)-1, if the component type is @c Object.
     */
    virtual int64_t getKeySerialVersionUID() const = 0;

    /**
     * Return the serial ID of the declared component type of this map or
     * @c NULL, if the component type is @c Object.
     */
    virtual const char *getComponentSerialID() const = 0;
    /**
     * Return the serialVersionUID of the declared component type of this map or
     * @c (int64_t)-1, if the component type is @c Object.
     */
    virtual int64_t getComponentSerialVersionUID() const = 0;

    virtual ObjPtr_less_t getLess() const = 0;

    virtual size_t getSize() const = 0;

    virtual bool containsKey(const ObjPtr<Object> &key) const = 0;

    virtual ObjPtr<Object> remove(const ObjPtr<Object> &key) = 0;

    virtual void put(const ObjPtr<Object> &key, const ObjPtr<Object> &value) = 0;

    virtual ObjPtr<Object> get(const ObjPtr<Object> &key) const = 0;

    /**
     * @return Whether this map contains a @c String key with the given cvalue.
     */
    virtual bool containsKey(const char *key) const = 0;

    /**
     * @return Whether this map contains a @c String key with the given cvalue.
     */
    virtual bool containsKey(const std::string &key) const = 0;

    /**
     * Remove and retrieve the value of the given @c String key.
     *
     * @return The value of the given key or @c null, if the value was
     *         @c null or the map did not contain the given key.
     */
    virtual ObjPtr<Object> remove(const char *key) = 0;

    /**
     * Remove and retrieve the value of the given @c String key.
     *
     * @return The value of the given key or @c null, if the value was
     *         @c null or the map did not contain the given key.
     */
    virtual ObjPtr<Object> remove(const std::string &key) = 0;

    /**
     * Add an entry with the given @c String key.
     */
    virtual void put(const char *key, const ObjPtr<Object> &value) = 0;

    /**
     * Add an entry with the given @c String key.
     */
    virtual void put(const std::string &key, const ObjPtr<Object> &value) = 0;

    /**
     * Add an entry with the given @c String key and @c String value.
     */
    virtual void putString(const std::string &key, const std::string &value) = 0;

    /**
     * Retrieve the value of the given @c String key.
     *
     * @return The value of the given key or @c null, if the value was @c null
     *         or the map did not contain the given key.
     */
    virtual ObjPtr<Object> get(const char *key) const = 0;

    /**
     * Retrieve the value of the given @c String key.
     *
     * @return The value of the given key or @c null, if the value was @c null
     *         or the map did not contain the given key.
     */
    virtual ObjPtr<Object> get(const std::string &key) const = 0;

    /**
     * Retrieve the value of the given @c String key.
     *
     * @return The value of the given key or an empty string, if the value was @c null
     *         or the map did not contain the given key.
     */
    virtual std::string getString(const char *key) const = 0;

    /**
     * Retrieve the value of the given @c String key.
     *
     * @return The value of the given key or an empty string, if the value was @c null
     *         or the map did not contain the given key.
     */
    virtual std::string getString(const std::string &key) const = 0;

    /**
     * Retrieve the value of the given @c String key.
     *
     * @return The value of the given key or @c defaultValue, if the value was @c null
     *         or the map did not contain the given key.
     */
    virtual std::string getString(const std::string &key, const std::string &defaultValue) const = 0;


    virtual ObjPtr<Iterator> getBeginIterator() = 0;
    virtual ObjPtr<Iterator> getEndIterator() = 0;
    virtual ObjPtr<Iterator> getLowerBound(const ObjPtr<Object> &o) = 0;

    virtual ObjPtr<Set> getKeySet() const = 0;
    virtual ObjPtr<List> getKeys() const = 0;
    virtual ObjPtr<List> getValues() const = 0;

    bool lessThan(const Object &o) const;
    bool equals(const Object &o) const;

    std::string toString() const;

    static ObjPtr<Map> newInstance();
    static ObjPtr<Map> newInstance(ObjPtr_less_t less);
    static ObjPtr<Map> newInstance(getSerialID_s_t getKeySerialID_s, getSerialVersionUID_s_t getKeySerialVersionUID_s,
                                   getSerialID_s_t getSerialID_s, getSerialVersionUID_s_t getSerialVersionUID_s);
    static ObjPtr<Map> newInstance(getSerialID_s_t getKeySerialID_s, getSerialVersionUID_s_t getKeySerialVersionUID_s,
                                   getSerialID_s_t getSerialID_s, getSerialVersionUID_s_t getSerialVersionUID_s,
                                   ObjPtr_less_t less);

    template <class K,class V> static ObjPtr<Map> newTypedInstance() {
        return Map::newInstance(K::getSerialID_s,K::getSerialVersionUID_s,
                                V::getSerialID_s,V::getSerialVersionUID_s);
    }

    template <class K,class V> static ObjPtr<Map> newTypedInstance(ObjPtr_less_t less) {
        return Map::newInstance(K::getSerialID_s,K::getSerialVersionUID_s,
                                V::getSerialID_s,V::getSerialVersionUID_s,less);
    }

protected:
    virtual ~Map();
};

}

#endif /* FANCYMAP_H_ */
