/***********************************************************
 * $Id: fancypollablechannel.h 1817 2017-01-27 17:39:29Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef POLLABLECHANNEL_H_
#define POLLABLECHANNEL_H_

extern "C" {
typedef struct fancylog_service_st fancylog_service;
}

#include <fancychannel.h>

namespace Fancy {

enum PollEvent {

    // these values match defines POLL* in poll.h resp. bits/poll.h
    PollEventNone     = 0x0000,
    PollEventIn       = 0x0001, // = POLLIN,   There is data to read
    PollEventPri      = 0x0002, // = POLLPRI,  There is urgent data to read
    PollEventOut      = 0x0004, // = POLLOUT,  Writing now will not block
    PollEventErr      = 0x0008, // = POLLERR,  Error condition
    PollEventHup      = 0x0010, // = POLLHUP,  Hung up
    PollEventNVal     = 0x0020  // = POLLNVAL, Invalid polling request
};

class FANCYBASE_API PollableChannel : public virtual Fancy::Channel {
public:

    typedef ObjPtr<PollableChannel> ptr_t;

    PollableChannel() {}

#ifdef WIN32
    /**
     * @return A pointer to an event handle used for asyncronous operations.
     */
    virtual void *getEventHandle() = 0;

    /**
     * @param events A pointer to an event mask, which is set when this function returns 1.
      *
     * @return 0, if the poll implementation should wait for @c *hEvent or 1,
     *         if there are pending poll events events, which are returned in events.
     */
    virtual int prePoll(PollEvent *events) = 0;

    /**
     *
     * @return 0, if no event occurred and 1 if there are pending poll events events,
     *         which are returned in events.
     */
    virtual int postPoll(PollEvent *events) = 0;
#else
    /**
     * @return The file desriptor to use for poll(2). This is usually the socket
     *         or file desriptor itself.
     */
    virtual int getPollFileDescriptor() const = 0;
#endif

    /**
     * Get the number of bytes, which may be read without blocking.
     */
    virtual size_t bytesAvailable() const = 0;

   /**
    * Poll this single pollable channel.
    *
    * @param events The returned event mask.
    * @param timeout The timeout in millseconds to wait for the next event.
    * @return 0, if no event occurred and 1 if there are pending poll events events,
     *         which are returned in events.
    */
    int poll (PollEvent *events, int timeout);

protected:
    virtual ~PollableChannel();
};

class ChannelPoller;

/**
 * An interface for handling poll events from a @c ChannelPoller.
 */
class FANCYBASE_API PollEventHandler : public virtual Object {

public:
    typedef ObjPtr<PollEventHandler> ptr_t;

    PollEventHandler() {}

    /**
     * Return a new instance, which logs the retrieved data to the
     * given log service using the given facility and priority.
     * pfx is used as a prefix for the logged text and may contain
     * some indicator of the source like "stdout", "stderr" or the
     * command, which produced the output.
     */
    static ptr_t newLoggingInstance(const std::string &pfx,
            const fancylog_service *svc,
            int facility,
            int priority);

    /**
     * Return a new instance, which logs the retrieved data to the
     * fancybase log service using the given facility and priority.
     * pfx is used as a prefix for the logged text and may contain
     * some indicator of the source like "stdout", "stderr" or the
     * command, which produced the output.
     */
    static ptr_t newLoggingInstance(const std::string &pfx,
                                    int facility,
                                    int priority);

    /**
    * Receive an event from a channel.
    *
    * An implementation is required to close the channel and call removeChannel
    * on context, when either a PollEventHup or PollEventErr is received or
    * PollEvntIn is received and read() returns 0.
    *
    * @param context The poller, which may be used to deregister.
    */
    virtual void handleEvent(const ObjPtr<ChannelPoller> &context,
                             const ObjPtr<PollableChannel> &channel, PollEvent event) = 0;

protected:
    virtual ~PollEventHandler();
};

/**
 * An object, which poll multiple file descriptors.
 */
class FANCYBASE_API ChannelPoller : public virtual Object {

public:
    typedef ObjPtr<ChannelPoller> ptr_t;

    /**
     * Return a new poller instance suitable for your current operating system.
     * This call is equivalent to calling @c newInstance(100), which means that the
     * returned implementation scales well for mid-sized servers. If you need an
     * implementation that scales to thousands or more channels, use
     * @c newInstance(size_t) directly.
     */
    static ptr_t newInstance();

    /**
     * Return a new poller instance suitable for your current operating system for
     * the given estimated number of file handles.
     *
     * The implementation returned may be different based on the given @c nfds parameter.
     * Under Linux, an implementation using poll or epoll may be returned depending on the
     * number of file descriptors requested.
     */
    static ptr_t newInstance(size_t nfds);

    ChannelPoller() {}

    /**
     * Add this channel to the list of channels to be watched.
     */
    virtual void addChannel(const PollableChannel::ptr_t& channel, const PollEventHandler::ptr_t &handler, PollEvent interest) = 0;

    /**
     * Set the events an application is interested in of an already registered channel.
     */
    virtual void setChannelInterest(const PollableChannel::ptr_t& channel, PollEvent interest) = 0;

    /**
     * Remove this channel from the list of channels to be watched.
     */
    virtual void removeChannel(const PollableChannel::ptr_t& channel) = 0;

    /**
     * Return the number of currently registered channels.
     */
    virtual size_t getChannelCount() const = 0;

    /**
     * Poll this collection of handles. A
     *
     * @param timeout The timeout in milliseconds to wait for the next event.
     * @return The number of handles which received an event.
     */
    virtual int poll (int timeout) = 0;

protected:
    virtual ~ChannelPoller();
};

}

#endif /* POLLABLECHANNEL_H_ */
