/***********************************************************
 * $Id: fancyprocess.h 1000 2013-10-07 16:59:28Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** Wrapper class for a subprocess started by our application.
**
*****************************************************************************/

#ifndef FANCYBASE_PROCESS_H
#define FANCYBASE_PROCESS_H

#include <fancyobject.h>

namespace Fancy {
  class FileHandle;

  class FANCYBASE_API Process : public virtual Fancy::Object
  {

  public:
    typedef ObjPtr<Process> ptr_t;

    static ptr_t newInstance();

    /**
     * Get the process ID of the current process.
     */
    static long getCurrentId();

    /**
     * @return Whether this object represents a process, which has been started using
     *         @c start() or @c fork().
     */
    virtual bool isStarted() const = 0;

    /**
     * @return Whether the destructor of this object will kill any started process,
     *         for which @c wait() has not yet been called, with signal 9.
     */
    virtual bool isKillOnDestruction() const = 0;

    /**
     * Set, whether the destructor of this object will kill any started process,
     * for which @c wait() has not yet been called, with signal 9.
     *
     * The default value is @c true, because this way programming errors do not lead
     * to left-over zombie processes. In programs, which fork other processes it may be
     * advisable to set this flag to @c false, because otherwise the process clone may
     * unexpectedly kill the spawned process.
     */
    virtual void setKillOnDestruction(bool b) = 0;

    /**
     * @return The process Id of the started process or @c 0, if this object does not represent
     *         a started process.
     */
    virtual long getId() const = 0;

    /**
     * Start a new subprocess.
     *
     * @param path The path of the executable to be executed.
     * @param argv A NULL-terminated array of command line arguments.
     *             Please note, that @c argv[0] must not be NULL, because
     *             this argument contains the name of the program, which is
     *             usually equal to @c path.
     * @param stdinHandle A reference to a handle, which will be set to a read-only
     *             pipe, which communicates to stdin of the new process.
     * @param stdoutHandle A reference to a handle, which will be set to a write-only
     *             pipe, which communicates to stdout of the new process.
     * @param stderrHandle A reference to a handle, which will be set to a write-only
     *             pipe, which communicates to stderr of the new process.
     */
    virtual void start(const char *path, const char *argv[],
               ObjPtr<FileHandle> &stdinHandle,
               ObjPtr<FileHandle> &stdoutHandle,
               ObjPtr<FileHandle> &stderrHandle ) = 0;

#ifndef WIN32
    /**
     * Fork a new subprocess under UNIX-like operating systems.
     *
     * @param stdinHandle A reference to a handle, which will be set to a read-only
     *             pipe, which communicates to stdin of the new process.
     * @param stdoutHandle A reference to a handle, which will be set to a write-only
     *             pipe, which communicates to stdout of the new process.
     * @param stderrHandle A reference to a handle, which will be set to a write-only
     *             pipe, which communicates to stderr of the new process.
     *
     * @return @c true, if we are in the child process,
     *         @c false, if we are in the parent process.
     */
    virtual bool fork(ObjPtr<FileHandle> &stdinHandle,
              ObjPtr<FileHandle> &stdoutHandle,
              ObjPtr<FileHandle> &stderrHandle) = 0;
#endif

    /**
     * Send a signal to a started process. Macros describing signal numbers can be found
     * inside the header @c fancysignal.h
     */
    virtual void kill(int signal) = 0;

    /**
     * Wait for the termination of a started process.
     *
     * This is a blocking operation, which may be interrupted by a signal,
     * which causes a @c CException to be thrown, which has the interrupted
     * flag set to @c true.
     *
     * @return The exit status of the finished process.
     */
    virtual int wait() = 0;
protected:
    Process();
    virtual ~Process();

  };

}

#endif
