/***********************************************************
 * $Id: fancyreflectiontarget.h 1000 2013-10-07 16:59:28Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYREFLECTIONTARGET_H_
#define FANCYREFLECTIONTARGET_H_

#include <fancyobject.h>

namespace Fancy {

class String;
template <class T> class ObjectArray;
class ReflectionTarget;

/**
 * The type of the static @reflectObject method, which is present in all
 * reflection-enabled interfaces.
 *
 * @param obj THe object, which is checked to be of the correct target type.
 * @return A reflection target, which call the interface methods on the given object.
 */
typedef ObjPtr<ReflectionTarget> (*reflectObject_t)(const Object::ptr_t &obj);

/**
 * The type of the static @c newProxyObject method, which is present in all
 * reflection-enabled interfaces.
 * @param target A reflected representation of the interface for which to
 *               return a proxy.
 * @return An new object, which implements the interfaces of the given target.
 */
typedef Object::ptr_t (*newProxyObject_t)(const ObjPtr<ReflectionTarget> &target);

/**
 * A small replacement interface for java's reflection mechanism.
 *
 * A reflection-enable interface of type @c T has the following static methods defined by
 * convention:
 *
 * @verbatim
 * static const char *getSerialID_s();
 * static ReflectionTarget::ptr_t reflectObject(const Object::ptr_t &obj);
 * static ReflectionTarget::ptr_t reflect (const T::ptr_t &impl);
 * static T::ptr_t newProxyInstance(const ReflectionTarget::ptr_t &target);
 * static Object::ptr_t newProxyObject(const ReflectionTarget::ptr_t &target);
 * @endverbatim
 *
 * The @c BeanInvocationHandlerFactory relies on these static functions in
 * the specified template functions, which register callback interfaces.
 *
 */
class FANCYBASE_API ReflectionTarget: public virtual Fancy::Object {
protected:
    ReflectionTarget();
    virtual ~ReflectionTarget();

public:
    typedef ObjPtr<ReflectionTarget> ptr_t;

    /**
     * Return the serial ID (aka the JAVA interface name) of the represented target.
     */
    virtual const char *getSerialID() const = 0;

    /**
     * Call a method of this target.
     *
     * @param method The name of the method to invoke.
     * @param args The argument of the method.
     */
    virtual Object::ptr_t invokeMethod(const char *method,
                                       const ObjPtr<ObjectArray<Object> > &args) = 0;

    /**
     * Return a NULL-terminated list of method names.
     *
     * @param nMethodsReturn If not NULL, the value of the variable pointed to by this parameter
     *                       will be set to the number of methods.
     */
    virtual const char * const *enumMethods(int *nMethodsReturn) const = 0;

    /**
     * Return a NULL-terminated list of method parameter serial IDs (aka JAVA class names).
     *
     * @param method The name of the method to query.
     * @param nMethodsReturn If not NULL, the value of the variable pointed to by this parameter
     *                       will be set to the number of parameters of the given method.
     */
    virtual const char * const *enumMethodParameterTypes(const char *method, int *nParamsReturn) const = 0;

    /**
     * Return the serial ID of the method's return type.
     * @param method The name of the method to query.
     */
    virtual const char *getMethodReturnType(const char *method) const = 0;
};

}

#endif /* FANCYREFLECTIONTARGET_H_ */
