/***********************************************************
 * $Id: fancysecuredigest.h 1750 2015-12-29 15:28:26Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas
**
** A class for creating message digests. IMplemented from scratch
** in order to calculate SHA-1/SHA-256 digests without openssl.
**
*****************************************************************************/

#ifndef FANCY_SECURE_DIGEST_H
#define FANCY_SECURE_DIGEST_H

#include <fancyarray.h>

namespace Fancy {

class Channel;

/**
 * Secure digests implemented from scratch.
 */
class FANCYBASE_API SecureDigest : public Object
{

public:
	typedef Fancy::ObjPtr<SecureDigest> ptr_t;

	/**
	 * Supported algorithm names are "SHA1", "SHA256" and
	 * the alternatives "SHA-1", "SHA-256"
	 *
	 * @return a new instance for the given algorithm name.
	 */
	static ptr_t newInstance(const char *algo);

	/**
	 * Clone a hash together with it's current state.
	 *
	 * @return a new instance for the given algorithm name.
	 */
	virtual ptr_t cloneState() const = 0;

	/**
	 * Reset the state of this secure digest.
	 */
	virtual void reset() = 0;

	/**
	 * Update the digest with the given binary data.
	 */
	virtual void addMessage(const void *m, size_t len) = 0;

	/**
	 * Update the digest with the given single byte given as integer.
	 */
	virtual void addByte(uint8_t byte) = 0;

	/**
	 * Update the digest with all remaining data from the given I/O channel.
	 */
	void addChannel(const Fancy::ObjPtr<Fancy::Channel> &channel);

	/**
	 * @return The name of the digest, which is @c "SHA1" or @c "SHA256"
	 */
	virtual std::string getAlgorithmName() const = 0;

	/**
	 * @return The length of the generated digests.
	 */
	virtual size_t getDigestSize() const = 0;

	/**
	 * return the value of the digest as an upper case hexadecimal string.
	 * The digest object is cleared and it may be reused by calling
	 * @c add*() and @getDigest() again.
	 */
	std::string getHexDigest();

	/**
	 * Return the value of the digest as a byte array.
	 * The digest object is cleared and it may be reused by calling
	 * @c add*() and @getDigest() again.
	 */
	Fancy::ArrayObject<int8_t>::ptr_t getDigest();

	/**
	 * Write the value of the digest to the memory location
	 * pointed to by @c out .
	 *
	 * The caller must assure, that there are at least
	 * @c getDigestSize() an exception is thrown.
	 *
	 * The digest object is cleared and it may be reused by calling
	 * @c add*() and @getDigest() again.
	 *
	 * @return The number of bytes written to @c out.
	 */
	virtual size_t getDigest(void *out) = 0;

protected:
	SecureDigest();
	virtual ~SecureDigest();

private:
	SecureDigest (const SecureDigest &a);
	SecureDigest &operator= (const SecureDigest &a);
};

}

#endif
