/***********************************************************
 * $Id: fancyserializable.h 1116 2014-01-14 22:38:55Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSERIALIZABLE_H_
#define FANCYSERIALIZABLE_H_

#include <fancyint.h>
#include <fancyobject.h>

namespace Fancy {

class Serializable;
class Serializer;

/**
 * The type of the static @c getSerialVersionUID_s() function declared for each Serializable
 * class by convention.
 */
typedef int64_t (*getSerialVersionUID_s_t)();

/**
 * The type of the static @c getOldSerialVersionUIDs_s() function declared for each Serializable
 * class by convention.
 *
 * @param nOldSVUIDs The content of the given pointer is set to the number of old serial version UIDs.
 * @return An array of known old serial version UIDs.
 */
typedef const int64_t *(*getOldSerialVersionUIDs_s_t)(int *nOldSVUIDs);

/**
 * The type of the static @c getSerialID_s() function declared for each Serializable
 * class by convention.
 */
typedef const char *(*getSerialID_s_t)();

/**
 * The prototype of the static @c newSerializableInstance() method present in all
 * serializable classes-
 */
typedef Fancy::ObjPtr<Serializable> (*newSerializableInstance_t)();

/*!
 * The interface for all C++ objects, which need to be written to or read from
 * a stream.
 *
 * By convention, a serializable class should declare two static functions to retrieve
 * the serialVersionUID and serialID without getting hold of an instance of the given class:
 *
 * @verbatim
 *  static int64_t getSerialVersionUID_s();
 *  static const int64_t *getOldSerialVersionUIDs_s(int *nOldSVUIDs);
 *  static const char *getSerialID_s();
 * @endverbatim
 *
 * Using this convention, the implementation of @c getSerialVersionUID() and @c getSerialID()
 * is a straightforward boilerplate task as shown in the UID example:
 *
 * @verbatim
 * int64_t UID::getSerialVersionUID_s() {
 *     return INT64_C(1086053664494604050);
 * }
 *
 * const char *UID::getSerialID_s() {
 *     return "java.rmi.server.UID";
 * }
 *
 * int64_t UID::getSerialVersionUID() const {
 *     return UID::getSerialVersionUID_s();
 * }
 *
 * const char *UID::getSerialID() const {
 * return UID::getSerialID_s();
 * }
 * @endverbatim
 */
class FANCYBASE_API Serializable: public virtual Fancy::Object {
public:
    typedef Fancy::ObjPtr<Serializable> ptr_t;

    /**
     * A fallback compatibility serial version UID query method, which
     * returns an empty list.
     */
    static const int64_t *getOldSerialVersionUIDs_s(int *nOldSVUIDs);

    /*!
     * Write the fields of the object to the given serializer.
     *
     * @param serializer The serializing stream to write to.
     */
    virtual void writeFields(const ObjPtr<Serializer> &serializer) const;

    /*!
     * Read the fields of the object from a deserializer and possibly replace the object by
     * a singleton instance.
     *
     * @param deserializer The serializer to read from.
     * @param serialVersionUID The serial version UID a read from the object header.
     *                         This parameter is passed in order to read a previous verion of
     *                         an object with an old serialVersionUID.
     * @return @c null, if this object is the final object, which is the case for most traditional
     *                POJO objects. Some objects like Enums or integral types may return a
     *                memory-optimized singleton as replacement for the original instance.
     *                The replacement possibilities should not be used by POJO classes or other
     *                types, because over-use of read-replacement causes major hazzles
     *                when the red instance i referenced from one of its fields.
     */
    virtual ObjPtr<Serializable> readFields(const ObjPtr<Serializer> &serializer, int64_t serialVersionUID);

protected:

    Serializable();
    virtual ~Serializable();
};

}

#endif /* FANCYSERIALIZABLE_H_ */
