/***********************************************************
 * $Id: fancyserializer.h 1134 2014-01-20 14:38:32Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSERIALIZER_H_
#define FANCYSERIALIZER_H_

#include <fancydatastream.h>

namespace Fancy {

template<class T> class ArrayObject;
class List;
class Map;
class SerializableFactory;
class EnumInfo;
class Channel;
class TypeConverter;
class JsonOptions;

class FANCYBASE_API Serializer: public virtual Fancy::Object {
public:
    typedef ObjPtr<Serializer> ptr_t;

    /**
     * Return a new default implementation instance, which reads from/writes to
     * the given stream.
     */
    static ptr_t newInstance(const DataStream::ptr_t& stream, const ObjPtr<SerializableFactory> &sf);

    /**
     * Return a new default implementation instance, which reads from/writes to
     * the given stream.
     *
     * This method allow to register a type converter, which might convert
     * "as serialized" objects to the required instances.
     */
    static ptr_t newInstance(const DataStream::ptr_t& stream,
    		const ObjPtr<SerializableFactory> &sf,
    		const ObjPtr<TypeConverter> &tc);

    /**
     * Return a new JSON implementation instance, which reads from/writes to
     * the given stream.
     *
     * The given SerializableFactory may be null in order to fall back to reading
     * objects as maps. This behavior is also implemented for all types unknown
     * to the serializable factory.
     *
     * This method allow to register a type converter, which might convert
     * "as serialized" objects to the required instances.
     *
     * A type converter is needed in order to forcibly cast number objects to
     * the required number object.
     *
     * Moreover, a type converted is needed in order to deserialize enum instances
     * serialized as strings.
     *
     * serializable objects, which are derived from teh requested target serialID
     * must contain as "serialID" property.
     */
    static ptr_t newJsonInstance(const ObjPtr<Channel>& channel,
    		const ObjPtr<SerializableFactory> &sf,
    		const ObjPtr<TypeConverter> &tc,
    		const ObjPtr<JsonOptions> &jo);

    /**
     * Return a new JSON implementation instance, which reads from/writes to
     * the given stream.
     *
     * The given SerializableFactory may be null in order to fall back to reading
     * objects as maps. This behavior is also implemented for all types unknown
     * to the serializable factory.
     *
     * This method intantiates a serializer, which does not bear a type converter
     * and may only return "as serialized" instances.
     *
     * Moreover, a type converted is needed in order to deserialize enum instances
     * serialized as strings.
     *
     * serializable objects, which are derived from teh requested target serialID
     * must contain as "serialID" property.
     */
    static ptr_t newJsonInstance(const ObjPtr<Channel>& channel,
    		const ObjPtr<SerializableFactory> &sf,
    		const ObjPtr<JsonOptions> &jo);

    virtual bool    readBoolean(const char *fieldName) = 0;
    virtual int8_t  readByte(const char *fieldName) = 0;
    virtual int16_t readShort(const char *fieldName) = 0;
    virtual int32_t readInt(const char *fieldName) = 0;
    virtual int64_t readLong(const char *fieldName) = 0;
    virtual wchar_t readChar(const char *fieldName) = 0;
    virtual float   readFloat(const char *fieldName) = 0;
    virtual double  readDouble(const char *fieldName) = 0;
    virtual std::string readUTF(const char *fieldName) = 0;
    virtual std::string readUTFLong(const char *fieldName) = 0;

    virtual void writeBoolean(const char *fieldName, bool b) = 0;
    virtual void writeByte   (const char *fieldName, int8_t i) = 0;
    virtual void writeShort  (const char *fieldName, int16_t i) = 0;
    virtual void writeInt    (const char *fieldName, int32_t i) = 0;
    virtual void writeLong   (const char *fieldName, int64_t i) = 0;
    virtual void writeChar   (const char *fieldName, wchar_t c) = 0;
    virtual void writeFloat  (const char *fieldName, float f) = 0;
    virtual void writeDouble (const char *fieldName, double d) = 0;
    virtual void writeUTF    (const char *fieldName, const char *s, int l=-1) = 0;
    virtual void writeUTFLong(const char *fieldName, const char *s, int l=-1) = 0;

    /**
     * Read object "as serialized" without any type conversion.
     *
     * This results in objects being deserialized as Fancy::Map instances for
     * JSON and other typeless protocols.
     *
     * this function is identical to calling
     * readObject(const char *fieldName, const char *targetSerialID)
     * with a null targetSerialID.
     */
    virtual ObjPtr<Object> readObject(const char *fieldName) = 0;

    virtual void writeObject (const char *fieldName, const ObjPtr<Object> &obj) = 0;

    /**
     * Read object erected using the given serialID.
     *
     * This results in objects being deserialized as an object for
     * JSON and other typeless protocols, if @c targetSerialID is not null.
     */
    virtual ObjPtr<Object> readObject(const char *fieldName, const char *targetSerialID) = 0;

    /**
     * Read a list of objects and erect the member objects using the given
     * serialID.
     *
     * This results in objects being deserialized as an object for
     * JSON and other typeless protocols, if @c memberSerialID is not null.
     */
    virtual ObjPtr<List> readObjectList(const char *fieldName, const char *memberSerialID) = 0;

    /**
      * Read an array of objects and erect the member objects using the given
      * serialID.
      *
      * This results in objects being deserialized as an object for
      * JSON and other typeless protocols, if @c memberSerialID is not null.
      *
      * @return An instance of ObjectArray<T>
      */
     virtual ObjPtr<List> readObjectArray(const char *fieldName, const char *memberSerialID) = 0;

     /**
      * Read an array of objects and erect the member objects using the given
      * list of expected serialIDs.
      *
      * This results in objects being deserialized as an object for
      * JSON and other typeless protocols, if @c memberSerialIDs is not null.
      *
      * @return An instance of ObjectArray<T>
      */
      virtual ObjPtr<List> readMixedObjectArray(const char *fieldName, const char * const *memberSerialIDs) = 0;

       /**
     * Read a map of objects erect the keys/values using the given
     * serialIDs.
     *
     * This results in objects being deserialized as an object for
     * JSON and other typeless protocols, if @c targetSerialID is not null.
     */
    virtual ObjPtr<Map> readObjectMap(const char *fieldName, const char *keySerialID, const char *valueSerialID) = 0;

    virtual ObjPtr<ArrayObject<bool> >    readBooleanArray(const char *fieldName) = 0;
    virtual ObjPtr<ArrayObject<int8_t> >  readByteArray(const char *fieldName) = 0;
    virtual ObjPtr<ArrayObject<int16_t> > readShortArray(const char *fieldName) = 0;
    virtual ObjPtr<ArrayObject<int32_t> > readIntArray(const char *fieldName) = 0;
    virtual ObjPtr<ArrayObject<int64_t> > readLongArray(const char *fieldName) = 0;
    virtual ObjPtr<ArrayObject<float> >   readFloatArray(const char *fieldName) = 0;
    virtual ObjPtr<ArrayObject<double> >  readDoubleArray(const char *fieldName) = 0;
    virtual ObjPtr<ArrayObject<wchar_t> > readCharacterArray(const char *fieldName) = 0;

    virtual void flush() = 0;

    virtual void shutdown(ShutdownOption o) = 0;

    virtual void close() = 0;

    virtual ObjPtr<SerializableFactory> getSerializableFactory() const = 0;

    virtual ObjPtr<TypeConverter> getTypeConverter() const = 0;

protected:
    Serializer();
    virtual ~Serializer();
};

}

#endif /* FANCYSERIALIZER_H_ */
