/***********************************************************
 * $Id: fancystring.h 1696 2015-07-01 12:48:21Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSTRING_H_
#define FANCYSTRING_H_

#include <fancyserializable.h>
#include <stdarg.h>

namespace Fancy {

/**
 * A base class for string implementation.
 * The default implementation is based on @c std::string, but
 * an alternative implementation may be provided and registered using registerImplementation().
 * For QT GUI programs, fancyqt4's Fancy::Qt4::String provide such an alternative implementation.
 *
 * Basically, throughout the fancy libraries it is assumed, that string instances
 * are UTF-8 encoded except in places where it is explicitly mentioned,
 * that the string is encoded in an 8bit platform encoding.
 *
 * Although not enforced by an abstract method in this header,
 * implementations are supposed to reimplement Fancy::Object's
 *   virtual void std::string toString();
 *
 */
class FANCYBASE_API String: public virtual Fancy::Serializable {
public:
    typedef ObjPtr<String> ptr_t;

    String() {}

    /**
     * If obj.isNull(), "" is returned.
     * Otherwise, a std::string representation of obj->toString() is returned.
     * This is almost equal to FANCY_OBJ_PTR_STRING, which returns "null" instead of "" for NULL objects.
     */
    static std::string toStdString(const Object::ptr_t &obj);

    /**
     * If obj.isNull(), "" is returned.
     * Otherwise, a std::wstring representation of obj->toString() is returned.
     */
    static std::wstring toStdWString(const Object::ptr_t &obj);

    static ObjPtr<Serializable> newSerializableInstance();

    /**
     * Return a reference to an empty string. This constructor is
     * mainly needed during a deserialization sequence, when
     * @c Serializable::readFields will be called subsequently.
     */
    static ObjPtr<String> newInstance();

    /**
     * Test the given C string and object pointer for equality.
     * This is an optimized version of
     * @verbatim
     * Fancy::fwd_equals(String::newInstance(s),o)
     * @endverbatim
     */
    static bool c_equals(const char *s, const ObjPtr<Object> &o);

    /**
     * Test the given object pointer and C string for equality.
     * This is an optimized version of
     * @verbatim
     * Fancy::fwd_equals(o,String::newInstance(s))
     * @endverbatim
     */
    static bool c_equals(const ObjPtr<Object> &o, const char *s);

    /**
     * Compare the given C string and object pointer.
     * This is an optimized version of
     * @verbatim
     * Fancy::fwd_less(String::newInstance(s),o)
     * @endverbatim
     */
    static bool c_less(const char *s, const ObjPtr<Object> &o);

    /**
     * Compare the given object pointer and C string.
     * This is an optimized version of
     * @verbatim
     * Fancy::fwd_equals(o,String::newInstance(s))
     * @endverbatim
     */
    static bool c_less(const ObjPtr<Object> &o, const char *s);

    /**
     * Return a new string instance representing the given UTF-8 string.
     * If @c s is NULL, a null reference is returned.
     */
    static ObjPtr<String> newInstance(const char *s);

    /**
      * Return a new string instance representing the given wide-character
      * string. The wide character string is converted to UTF-8 by this method.
      */
     static ObjPtr<String> newInstance(const wchar_t *s);

     /**
     * Register the implementation returned by all @c newInstance functions.
     * If not registered here, a default implementation based on @c std::string
     * is registered.
     *
     * This function mainly exists in order to make GUI libraries like Qt play well
     * with fancylibs. The library @c fancyqt4 provides for a more sophisticated
     * @c String implementation, which stores a @c QString and a cached utf-8
     * representation based .
     */
    static void registerImplementation(String *(*newString_1)(),
                                       String *(*newString_2)(const char *),
                                       String *(*newString_3)(const std::string &));

    /**
     * Return a new string instance representing the given UTF-8 string.
     */
    static ObjPtr<String> newInstance(const std::string &s);

    /**
     * Return a new string instance representing the given wide-character
     * string. The wide character string is converted to UTF-8
     */
    static ObjPtr<String> newInstance(const std::wstring &s);

    /**
     * Return a new string instance representing the given UTF-8 string.
     * This version always return the internal @c std::string default
     * implementation regardless, which implementation has been registered
     * using @c registerImplementation.
     *
     * This method is of limited interest for normal users. However, it might
     * prove useful in @c String implementations, which internally cache
     * a @c std::string instance.
     */
    static ObjPtr<String> newStdInstance(const std::string &s);

    /**
     * Format a string using a printf-style format string and format
     * arguments.
     */
    #ifdef __GNUG__
      __attribute__((format(printf,1,2)))
#endif
    static ObjPtr<String> newFmtInstance(const char *fmt, ...);

    /**
     * Format a string using a printf-style format string and a stdarg
     * argument list like in @c vsnprintf.
     */
    static ObjPtr<String> newVFmtInstance(const char *fmt, va_list ap);

    /**
     * Format a @c std::string using a printf-style format string and stdarg
     * argument list like in @c vsnprintf.
     */
#ifdef __GNUG__
      __attribute__((format(printf,1,2)))
#endif
    static std::string sprintf(const char *fmt, ...);

    /**
     * Format a @c std::string using a printf-style format string and a
     * arguments.
     */
    static std::string vsprintf(const char *fmt, va_list ap);

    /**
     * The minimum value of a
     * <a href="http://www.unicode.org/glossary/#high_surrogate_code_unit">
     * Unicode high-surrogate code unit</a>
     * in the UTF-16 encoding, constant {@code '\u005CuD800'}.
     * A high-surrogate is also known as a <i>leading-surrogate</i>.
     *
     * Taken from java.lang.Character
     */
    static const int MIN_HIGH_SURROGATE = 0xD800;

    /**
     * The maximum value of a
     * <a href="http://www.unicode.org/glossary/#high_surrogate_code_unit">
     * Unicode high-surrogate code unit</a>
     * in the UTF-16 encoding, constant {@code '\u005CuDBFF'}.
     * A high-surrogate is also known as a <i>leading-surrogate</i>.
     *
     * Taken from java.lang.Character
     */
    static const int MAX_HIGH_SURROGATE = 0xDBFF;

    /**
     * The minimum value of a
     * <a href="http://www.unicode.org/glossary/#low_surrogate_code_unit">
     * Unicode low-surrogate code unit</a>
     * in the UTF-16 encoding, constant {@code '\u005CuDC00'}.
     * A low-surrogate is also known as a <i>trailing-surrogate</i>.
     *
     * Taken from java.lang.Character
     */
    static const int MIN_LOW_SURROGATE  = 0xDC00;

    /**
     * The maximum value of a
     * <a href="http://www.unicode.org/glossary/#low_surrogate_code_unit">
     * Unicode low-surrogate code unit</a>
     * in the UTF-16 encoding, constant {@code '\u005CuDFFF'}.
     * A low-surrogate is also known as a <i>trailing-surrogate</i>.
     *
     * Taken from java.lang.Character
     */
    static const int MAX_LOW_SURROGATE  = 0xDFFF;

    /**
     * The minimum value of a
     * <a href="http://www.unicode.org/glossary/#supplementary_code_point">
     * Unicode supplementary code point</a>, constant {@code U+10000}.
     *
     * Taken from java.lang.Character
     */
    static const int  MIN_SUPPLEMENTARY_CODE_POINT = 0x010000;

    /**
     * The maximum value of a
     * <a href="http://www.unicode.org/glossary/#code_point">
     * Unicode code point</a>, constant {@code U+10FFFF}.
     *
     * Taken from java.lang.Character
     */
    static const int MAX_CODE_POINT = 0x10FFFF;

    /**
     * Convert a wide-character string to an UTF-8 std::String instance.
     * This takes the replacement of UTF-16 surrogate pairs into account.
     */
    static std::string wstrtoutf8(const std::wstring &s);

    /**
     * Convert a null-terminated wide-character string to an UTF-8
     * std::string instance. If a null pointer is passed, an empty
     * string is returned.
     * This takes the replacement of UTF-16 surrogate pairs into account.
     */
    static std::string wstrtoutf8(const wchar_t *s);

    /**
     * Convert the given UTF-8 string to a wide character string.
     */
    static std::wstring utf8towstr(const std::string &s);

    /**
     * Get the UTF8-representation of this string, which is the same string as
     * return by @c toString().
     */
    virtual std::string getUtf8String() const = 0;

    /**
     * Return the length of the UTF-8 represenation of this string, which is
     * the same as @c getUtf8String().length() or @c toString().length().
     */
    virtual size_t getUtf8Length() const = 0;

    static int64_t getSerialVersionUID_s();
    static const char *getSerialID_s();

    /**
     * @return A pseudo serial version UID, which is passed to @c readFields, when
     *         the string contetn should be read in the long format by using
     *         @c DataChannel.readUTFLong().
     */
    static int64_t getLongSerialVersionUID_s();

    int64_t getSerialVersionUID() const;
    const char *getSerialID() const;

    // you may want to look at fancystringhelper, too

protected:
    virtual ~String();
};

}

#endif /* FANCYSTRING_H_ */
