/***********************************************************
 * $Id: fancythrowable.h 1107 2014-01-08 09:03:23Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYTHROWABLE_H_
#define FANCYTHROWABLE_H_

#include <fancyserializable.h>

namespace Fancy {

class String;
class StackTraceElement;
template <class T> class ObjectArray;

/**
 * A representation of the java class @c java.lang.Throwable
 */
class FANCYBASE_API Throwable: public virtual Fancy::Serializable {
protected:
    Throwable();
    virtual ~Throwable();

    std::string getClassName() const;

public:
    typedef ObjPtr<Throwable> ptr_t;

    static Serializable::ptr_t newSerializableInstance();
    static ptr_t newInstance();
    static ptr_t newInstance(const ObjPtr<String> &detailMessage);
    static ptr_t newInstance(const ObjPtr<String> &detailMessage,
                             const Throwable::ptr_t &cause);

    static int64_t getSerialVersionUID_s();
    static const char *getSerialID_s();

    int64_t getSerialVersionUID() const;
    const char *getSerialID() const;

    void writeFields(const ObjPtr<Serializer> &serializer) const;
    ObjPtr<Serializable> readFields(const ObjPtr<Serializer> &serializer, int64_t serialVersionUID);

    std::string toString() const;

    virtual void logWithStackTrace(std::ostream &os) const;

    Throwable::ptr_t getCause() const;
    void setCause(const Throwable::ptr_t &cause);

    ObjPtr<String> getDetailMessage() const;
    void setDetailMessage(const ObjPtr<String> &msg);

    ObjPtr<ObjectArray<StackTraceElement> > getStackTrace() const;
    void setStackTrace(const ObjPtr<ObjectArray<StackTraceElement> > &stackTrace);

private:
    Throwable::ptr_t cause;
    ObjPtr<String> detailMessage;
    ObjPtr<ObjectArray<StackTraceElement> > stackTrace;
};

/**
 * A representation of the java class @c java.lang.Exception
 */
class FANCYBASE_API JavaException: public Throwable {
protected:
    JavaException();
    virtual ~JavaException();

public:
    typedef ObjPtr<JavaException> ptr_t;

    static Serializable::ptr_t newSerializableInstance();
    static ptr_t newInstance();
    static ptr_t newInstance(const ObjPtr<String> &detailMessage);
    static ptr_t newInstance(const ObjPtr<String> &detailMessage,
                             const Throwable::ptr_t &cause);

    static int64_t getSerialVersionUID_s();
    static const char *getSerialID_s();
    static const char *getSuperSerialID_s();

    int64_t getSerialVersionUID() const;
    const char *getSerialID() const;
    const char *getSuperSerialID() const;
};

/**
 * A representation of the java class @c java.lang.RuntimeException
 */
class FANCYBASE_API JavaRuntimeException: public Throwable {
protected:
    JavaRuntimeException();
    virtual ~JavaRuntimeException();

public:
    typedef ObjPtr<JavaRuntimeException> ptr_t;

    static Serializable::ptr_t newSerializableInstance();
    static ptr_t newInstance();
    static ptr_t newInstance(const ObjPtr<String> &detailMessage);
    static ptr_t newInstance(const ObjPtr<String> &detailMessage,
                             const Throwable::ptr_t &cause);

    static int64_t getSerialVersionUID_s();
    static const char *getSerialID_s();
    static const char *getSuperSerialID_s();

    int64_t getSerialVersionUID() const;
    const char *getSerialID() const;
    const char *getSuperSerialID() const;
};

/**
 * A representation of the java class @c org.clazzes.remoting.FlatRemoteException
 */
class FANCYBASE_API FlatRemoteException: public JavaException {
protected:
    FlatRemoteException();
    virtual ~FlatRemoteException();

    std::string getClassName() const;

public:
    typedef ObjPtr<FlatRemoteException> ptr_t;

    static Serializable::ptr_t newSerializableInstance();
    static ptr_t newInstance();
    static ptr_t newInstance(const ObjPtr<String> &originalClassName);
    static ptr_t newInstance(const ObjPtr<String> &originalClassName,
                             const ObjPtr<String> &detailMessage);
    static ptr_t newInstance(const ObjPtr<String> &originalClassName,
                             const ObjPtr<String> &detailMessage,
                             const Throwable::ptr_t &cause);

    static int64_t getSerialVersionUID_s();
    static const char *getSerialID_s();
    static const char *getSuperSerialID_s();

    int64_t getSerialVersionUID() const;
    const char *getSerialID() const;
    const char *getSuperSerialID() const;

    void writeFields(const ObjPtr<Serializer> &serializer) const;
    ObjPtr<Serializable> readFields(const ObjPtr<Serializer> &serializer, int64_t serialVersionUID);

    ObjPtr<String> getOriginalClassName() const;
    void setOriginalClassName(const ObjPtr<String> &className);

private:
    ObjPtr<String> originalClassName;
};


/**
 * A java stack trace element which is part of a Throwable.
 */
class FANCYBASE_API StackTraceElement: public virtual Fancy::Serializable {
protected:
    StackTraceElement();
    virtual ~StackTraceElement();

public:
    typedef ObjPtr<StackTraceElement> ptr_t;

    static Serializable::ptr_t newSerializableInstance();
    static ptr_t newInstance(const ObjPtr<String> &declaringClass,
                             const ObjPtr<String> &methodName,
                             const ObjPtr<String> &fileName,
                             int32_t lineNumber);

    std::string toString() const;
    void log (std::ostream &os) const;

    static int64_t getSerialVersionUID_s();
    static const char *getSerialID_s();

    int64_t getSerialVersionUID() const;
    const char *getSerialID() const;

    void writeFields(const ObjPtr<Serializer> &serializer) const;
    ObjPtr<Serializable> readFields(const ObjPtr<Serializer> &serializer, int64_t serialVersionUID);

    ObjPtr<String> getDeclaringClass() const;
    ObjPtr<String> getMethodName() const;
    ObjPtr<String> getFileName() const;
    int32_t        getLineNumber() const;

    void setDeclaringClass(const ObjPtr<String> &declaringClass);
    void setMethodName(const ObjPtr<String> &methodName);
    void setFileName(const ObjPtr<String> &fileName);
    void setLineNumber(int32_t lineNumber);

private:
    ObjPtr<String> declaringClass;
    ObjPtr<String> methodName;
    ObjPtr<String> fileName;
    int32_t lineNumber;
};

}

#endif /* FANCYTHROWABLE_H_ */
