/***********************************************************
 * $Id: fancytimer.h 1000 2013-10-07 16:59:28Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYTIMER_H_
#define FANCYTIMER_H_

#include <fancyrunnable.h>

namespace Fancy {

/**
 * A Timer executes tasks in the future.
 */
class FANCYBASE_API Timer: public Fancy::Runnable {
protected:
    Timer();
    virtual ~Timer();
public:
    typedef ObjPtr<Timer> ptr_t;

    /**
     * Return a new timer instance. The returned instance has to be assigned to a thread
     * in order to perform @c run() asynchronously.
     */
    static ptr_t newInstance();

    /**
     * Run the given runnable at the given point in time in a static instance, which
     * splits of a timer thread the first time called.
     *
     * @param millis Number of milliseconds since the epoch 1970-01-01T00:00:00Z
     *               when the runnable shall be executed.
     * @param runnable The task to perform.
     */
    static void runAt_s(uint64_t millis, const Runnable::ptr_t &runnable);

    /**
     * Run the given runnable at the given point in time from now in a static instance, which
     * splits of a timer thread the first time called.
     *
     * @param delta Number of milliseconds from now,
     *              when the runnable shall be executed.
     * @param runnable The task to perform.
     */
    static void runIn_s(unsigned delta, const Runnable::ptr_t &runnable);

    /**
     * Execute the currently running task in a static instance
     * and subsequently stop the workerthread of the static instance.
     */
    static void stop_s();

    /**
     * Run the given runnable at the given point in time.
     *
     * @param millis Number of milliseconds since the epoch 1970-01-01T00:00:00Z
     *               when the runnable shall be executed.
     * @param runnable The task to perform.
     */
    virtual void runAt(uint64_t millis, const Runnable::ptr_t &runnable) = 0;

    /**
     * Run the given runnable at the given point in time from now.
     *
     * @param delta Number of milliseconds from now,
     *               when the runnable shall be executed.
     * @param runnable The task to perform.
     */
    virtual void runIn(unsigned delta, const Runnable::ptr_t &runnable) = 0;

    /**
     * Execute the currently running task and subsequently exit from @c run().
     */
    virtual void stop() = 0;
};

}

#endif /* FANCYTIMER_H_ */
