/***********************************************************
 * $Id: fancytimeval.h 1000 2013-10-07 16:59:28Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas/EV-i
**
** Convenience C++ operators on struct timeval from sys/time.h.
**
*****************************************************************************/

#ifndef FANCY_TIMEVAL_H
#define FANCY_TIMEVAL_H 1

#include <sys/time.h>
#include <fancybase.h>
#include <fancyint.h>

namespace Fancy {

/** returns the milliseconds since epoche, i.e. 0 for 1.1.1970 GMT */
uint64_t FANCYBASE_API getCurrentTimeMillis();

/** returns the milliseconds since epoche and sets tz_off in minutes.
 *  In central europe, tz_off becomes 60 in winter, 120 in summer.
 *  The results are suitable for Fancy::UtcTimestamp::newInstance(uint64_t millis, int tz_off)
 */
uint64_t FANCYBASE_API getCurrentTimeMillisTZ(int *tz_off);

/** returns the tz offset of the local timezone effective at the given milliseconds since the epoch.
 * In central europe, the return value becomes 60 in winter, 120 in summer.
 */
int FANCYBASE_API getLocalTZOffForUTCMillis(uint64_t millis);

/** returns the basic tz offset of the local timezone.
 * In central europe, the return value is 60 both in winter and summer.
 */
int FANCYBASE_API getLocalTZOff();


}

// convenience operators on struct timeval, mainly a cut'n'paste
// from the C-macros in sys/time.h Since these macros are not always
// included, we make C++-inlines from them and put it here.
// (Under glibc-2.2, you have to #define _BSD_SOURCE cf. to gettimeofday(2)
//  and under other UNICES, I suspect them to be completely missing.)

inline bool operator < (const struct timeval &a, const struct timeval &b)
{
  return
    (a.tv_sec == b.tv_sec)  ?
    (a.tv_usec < b.tv_usec) :
    (a.tv_sec < b.tv_sec)   ;
}

inline bool operator > (const struct timeval &a, const struct timeval &b)
{
  return
    (a.tv_sec == b.tv_sec)  ?
    (a.tv_usec > b.tv_usec) :
    (a.tv_sec > b.tv_sec)   ;
}

inline bool operator >= (const struct timeval &a, const struct timeval &b)
{
  return
    (a.tv_sec == b.tv_sec)  ?
    (a.tv_usec >= b.tv_usec) :
    (a.tv_sec > b.tv_sec)   ;
}

inline bool operator <= (const struct timeval &a, const struct timeval &b)
{
  return
    (a.tv_sec == b.tv_sec)  ?
    (a.tv_usec <= b.tv_usec) :
    (a.tv_sec < b.tv_sec)   ;
}

inline struct timeval &operator+=(struct timeval &a, const struct timeval &b)
{
  a.tv_sec  += b.tv_sec;
  a.tv_usec += b.tv_usec;

  if (a.tv_usec >= 1000000) {
    ++a.tv_sec;
    a.tv_usec -= 1000000;
  }
  return a;
}

inline struct timeval &operator -= (struct timeval &a, const struct timeval &b)
{
  a.tv_sec -= b.tv_sec;
  a.tv_usec -= b.tv_usec;
  if (a.tv_usec < 0) {
    --a.tv_sec;
    a.tv_usec += 1000000;
  }
  return a;
}

inline int tv_to_msec (const struct timeval &a)
{
  return a.tv_sec*1000 + (a.tv_usec+500) / 1000;
}

inline void tv_set_msec (struct timeval &a, unsigned m)
{
  a.tv_sec=m/1000;
  a.tv_usec=(m%1000)*1000;
}

#endif // ! FANCYSOCK_TIMEVAL_H
