/***********************************************************
 * $Id: fancytypeconverter.h 1206 2014-03-12 09:38:22Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYTYPECONVERTER_H_
#define FANCYTYPECONVERTER_H_

#include <fancyobject.h>

namespace Fancy {

class List;
template<class T> class ArrayObject;

/*!
 * The interface for a converting objects.
 */
class FANCYBASE_API TypeConverter: public virtual Fancy::Object {
public:
    typedef Fancy::ObjPtr<TypeConverter> ptr_t;

    typedef ObjPtr<Object> (*convertFromStdString_t)(const std::string &);

    typedef ObjPtr<Object> (*convertFromObjPtr_t)(const ObjPtr<Object> &);

    template<class T> static ObjPtr<Object> stdStringCtorAdapter(const std::string &s) {
    	return T::newInstance(s);
    }

    template<class Target, class Source> static ObjPtr<Object> sourceCtorAdapter(const ObjPtr<Object> &o) {
     	return Target::newInstance(ObjPtr<Source>(o));
    }

    /**
     * Return a new default implementation. The returned instance is aware of all
     * number classes in the library @c fancybase.
     */
    static ptr_t newInstance();

    /**
     * Return a new default implementation using the given delegate for resolving unknown
     * serial IDs.
     *
     * The returned instance is not aware of any class in the library @c fancybase.
     */
    static ptr_t newInstance(const ptr_t &delegate);

    /*!
     *
     * @param targetSerialID The target serial ID by \c Serializable::getSerialID().
     *
     * @param serialVersionUID The object version ID as returned by
     *                         \c Serializable::getSerialVersionUID().
     * @return a new object of the given target type id. If the o is of type Fancy::String, all
     *         registered stdString converters are additionally considered.
     */
    virtual ObjPtr<Object> convertObject(const char *targetSerialID, const ObjPtr<Object> &o) const = 0;

    virtual ObjPtr<ArrayObject<bool> >    convertToBooleanArray(const ObjPtr<List> &o) const = 0;
    virtual ObjPtr<ArrayObject<int8_t> >  convertToByteArray(const ObjPtr<List> &o) const = 0;
    virtual ObjPtr<ArrayObject<int16_t> > convertToShortArray(const ObjPtr<List> &o) const = 0;
    virtual ObjPtr<ArrayObject<int32_t> > convertToIntArray(const ObjPtr<List> &o) const = 0;
    virtual ObjPtr<ArrayObject<int64_t> > convertToLongArray(const ObjPtr<List> &o) const = 0;
    virtual ObjPtr<ArrayObject<float> >   convertToFloatArray(const ObjPtr<List> &o) const = 0;
    virtual ObjPtr<ArrayObject<double> >  convertToDoubleArray(const ObjPtr<List> &o) const = 0;
    virtual ObjPtr<ArrayObject<wchar_t> > convertToCharacterArray(const ObjPtr<List> &o) const = 0;

    virtual bool    convertToBoolean(const ObjPtr<Object> &o) const = 0;
    virtual int8_t  convertToByte(const ObjPtr<Object> &o) const = 0;
    virtual int16_t convertToShort(const ObjPtr<Object> &o) const = 0;
    virtual int32_t convertToInt(const ObjPtr<Object> &o) const = 0;
    virtual int64_t convertToLong(const ObjPtr<Object> &o) const = 0;
    virtual float   convertToFloat(const ObjPtr<Object> &o) const = 0;
    virtual double  convertToDouble(const ObjPtr<Object> &o) const = 0;
    virtual wchar_t convertToCharacter(const ObjPtr<Object> &o) const = 0;

    virtual ObjPtr<Object> convertString(const char *targetSerialID, const std::string &s) const = 0;

    /**
     * Register a convert from the given source serialID to a target serialID.
     */
    virtual void registerConverter(const char *targetSerialID, const char *sourceSerialID,
    		convertFromObjPtr_t converter) = 0;

    /**
     * Register a converter from std::string or Fancy::String to a given target
     * type, which has a T::newInstance(const std::String &s) method.
     * This is escpecially the case with all generated enum objects,
     * which should be registered using this function, when they are serialized
     * as a string.
     */
    virtual void registerStdStringConverter(const char *targetSerialID,
    		convertFromStdString_t converter) = 0;

    /**
     * Register a converter from @c Source to @c Target by determining the serial IDs
     * from the static getSerialID_s() functions.
     */
    template <class Target, class Source> void registerConverterT(convertFromObjPtr_t converter) {
        this->registerConverter(Target::getSerialID_s(),Source::getSerialID_s(),converter);
    }

    /**
     * Register a converter from @c Source to @c Target by determining the serial IDs
     * from the static getSerialID_s() functions and converting the values
     * by Target::newInstance(const Source::ptr_t &s) constructors.
     */
    template <class Source, class Target> void registerSourceCtorT() {
        this->registerConverter(Source::getSerialID_s(),Target::getSerialID_s(),sourceCtorAdapter<Source,Target>);
    }

    /**
     * Register a converter from @c std::string or @c Fancy::String to
     * @c Target by determining the serial IDs
     * from the static getSerialID_s() functions and converting the values
     * by the Target::newInstance(const std::string &s) constructor.
     * This is especially useful for deserializing Enum values as string.
     */
    template <class Target> void registerStdStringCtorT() {
        this->registerStdStringConverter(Target::getSerialID_s(),stdStringCtorAdapter<Target>);
    }

protected:
    TypeConverter();
    ~TypeConverter();
};

}

#endif /* FANCYSERIALIZABLE_H_ */
