/***********************************************************
 * $Id: fancyuuid.h 1004 2013-10-08 16:57:04Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYUUID_H_
#define FANCYUUID_H_

#include <fancyserializable.h>

namespace Fancy {

/**
 * A universally unique identifier, see http://en.wikipedia.org/wiki/Universally_unique_identifier
 */
class FANCYBASE_API UUID: public virtual Fancy::Serializable {

    UUID(uint64_t m, uint64_t l);
    UUID();

public:
    typedef ObjPtr<UUID> ptr_t;

    /**
     * Return an uninitialized instance, which will be initialized through
     * @c Serializable::readFields().
     */
    static ObjPtr<Serializable> newSerializableInstance();

    /**
     * Return an instance, which is initialized as a type 4 (pseudo randomly generated)
     * UUID.
     */
    static ptr_t newInstance();

    /**
      * Return an UUID instance from a string representation of type
      * @c xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx.
      */
    static ptr_t newInstance(const std::string &s);

    /**
      * Return an UUID instance from the given binary representation.
      *
      * @param mostSigBits The 64 most significant bits of the new UUID.
      * @param leastSigBits The 64 least significant bits of the new UUID.
      */
    static ptr_t newInstance(uint64_t mostSigBits, uint64_t leastSigBits);

	// override.
	std::string toString() const;

	bool operator== (const UUID &a) const;
	bool operator!= (const UUID &a) const;
    bool operator<  (const UUID &a) const;

    bool lessThan(const Object &o) const;
    bool equals(const Object &o) const;

    /**
     * Return the 64 least significant bits of this UUID.
     */
    uint64_t getLeastSigBits()   const { return this->leastSigBits; }

    /**
     * Return the 64 most significant bits of this UUID.
     */
    uint64_t getMostSigBits()  const { return this->mostSigBits; }

    /**
     * The version number associated with this {@code UUID}.  The version
     * number describes how this {@code UUID} was generated.
     *
     * The version number has the following meaning:
     * <p><ul>
     * <li>1    Time-based UUID
     * <li>2    DCE security UUID
     * <li>3    Name-based UUID
     * <li>4    Randomly generated UUID
     * </ul>
     *
     * @return  The version number of this {@code UUID}
     */
    int getVersion() const;

    /**
      * The variant number associated with this {@code UUID}.  The variant
      * number describes the layout of the {@code UUID}.
      *
      * The variant number has the following meaning:
      * <p><ul>
      * <li>0    Reserved for NCS backward compatibility
      * <li>2    The Leach-Salz variant (used by this class)
      * <li>6    Reserved, Microsoft Corporation backward compatibility
      * <li>7    Reserved for future definition
      * </ul>
      *
      * @return  The variant number of this {@code UUID}
      */
    int getVariant() const;

    // implementation of Fancy::Serializable
    static int64_t getSerialVersionUID_s();
    static const char *getSerialID_s();

    int64_t getSerialVersionUID() const;
    const char *getSerialID() const;
    void writeFields(const ObjPtr<Serializer> &serializer) const;
    ObjPtr<Serializable> readFields(const ObjPtr<Serializer> &serializer, int64_t serialVersionUID);


protected:
	~UUID();

private:
	// this type is immutable.
	UUID &operator=(const UUID &);

	uint64_t leastSigBits;
	uint64_t mostSigBits;
};

}

#endif /* FANCYUID_H_ */
